Graph

Struct Graph 

pub struct Graph<V = (), E = ()>
where V: Default + Clone, E: Default + Clone,
{ pub nodes: Vec<V>, pub head: Vec<usize>, pub edges: Vec<(usize, usize, E)>, /* private fields */ }
Graph representation using adjacency list.

V is the information stored in each node, and E is the information stored in each edge.

Fields

nodes: Vec<V>

The information stored in each node.

head: Vec<usize>

The head node in adjacency list for each node in graph.

edges: Vec<(usize, usize, E)>

The information stored in each edge.

Implementations

Source

impl<V, E> Graph<V, E>
where V: Default + Clone, E: Default + Clone,

pub fn new(n: usize) -> Self

Create a new graph with n nodes.

pub fn from_nodes(nodes: Vec<V>) -> Self

Create a new graph from a given vector of nodes.

pub fn empty() -> Self

pub fn len_nodes(&self) -> usize

Get the number of nodes in the graph.

pub fn len_edges(&self) -> usize

Get the number of edges in the graph.

pub fn erase_edge(&mut self, idx: &mut usize)

Erase the edge with index idx.

pub fn sort_edges(&mut self, node: usize)

Sort the edges of a node by the node id.

pub fn sort_edges_by<F>(&mut self, node: usize, compare: F)
where F: FnMut(&(usize, &V), &(usize, &V)) -> Ordering,

Sort the edges of a node by the given comparator.

pub fn add_edge(&mut self, from: usize, to: usize, info: E)

Add an undirected edge between from and to with information info.

pub fn get_edges_from(&self, edge: usize) -> impl Iterator<Item = (&usize, &E)>

Returns an iterator over the edges from the edge with index edge. The iterator returns the destination node, and the information stored in the edge.

pub fn get_edges_enum_from( &self, edge: usize, ) -> impl Iterator<Item = (usize, (&usize, &E))>

Returns an iterator over the edges from the edge with index edge. The iterator returns the index of the edge, the destination node, and the information stored in the edge.

pub fn get_edges(&self, node: usize) -> impl Iterator<Item = (&usize, &E)>

Returns an iterator over the edges of a node. The iterator returns the destination node, and the information stored in the edge.

pub fn get_edges_enum( &self, node: usize, ) -> impl Iterator<Item = (usize, (&usize, &E))>

Returns an iterator over the edges of a node. The iterator returns the index of the edge, the destination node, and the information stored in the edge.

pub fn get_edges_from_once<'a>( &'a self, cur: &'a mut usize, ) -> impl Iterator<Item = (&usize, &E)> + 'a

Returns an iterator over the edges of a node. The iterator returns the destination node, and the information stored in the edge. This is used if you want to iterate edges only once.

pub fn get_edges_enum_from_once<'a>( &'a self, cur: &'a mut usize, ) -> impl Iterator<Item = (usize, (&usize, &E))> + 'a

Returns an iterator over the edges of a node. The iterator returns the index of the edge, the destination node, and the information stored in the edge. This is used if you want to iterate edges only once.

pub fn get_edge(&self, idx: usize) -> (usize, usize, &E)

Get the edge information of the edge with index idx.

pub fn get_twin_edge(&self, idx: usize) -> (usize, usize, &E)

Get the twin edge information of the edge with index idx. Often used in undirected graphs as the reverse edge.

pub fn get_edge_mut(&mut self, idx: usize) -> (usize, usize, &mut E)

Get the edge information of the edge with index idx.

pub fn get_twin_edge_mut(&mut self, idx: usize) -> (usize, usize, &mut E)

Get the twin edge information of the edge with index idx. Often used in undirected graphs as the reverse edge.